@@ -225,7 +225,13 @@ class AuthInfo(object):
         :returns: list of auth method names
 
         """
-        return self.auth['identity']['methods'] or []
+        # Sanitizes methods received in request's body
+        # Filters out duplicates, while keeping elements' order.
+        method_names = []
+        for method in self.auth['identity']['methods']:
+            if method not in method_names:
+                method_names.append(method)
+        return method_names
 
     def get_method_data(self, method):
         """Get the auth method payload.
@@ -354,6 +360,13 @@ class Auth(controller.V3Controller):
             msg = _('User not found')
             raise exception.Unauthorized(msg)
 
+        # NOTE(nkinder): We only log this message in Havana.  In Icehouse and
+        # later, we emit a CADF notification upon successful token issuance
+        # instead.
+        msg = _('Issued token for user %(u_id)s')
+        msg = msg % {'u_id': auth_context['user_id']}
+        LOG.info(msg)
+
     @controller.protected()
     def check_token(self, context):
         token_id = context.get('subject_token_id')
