@@ -1,9 +1,9 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-# Copyright 2012 OpenStack LLC
+# Copyright 2012 OpenStack Foundation
 # Copyright 2010 United States Government as represented by the
 # Administrator of the National Aeronautics and Space Administration.
-# Copyright 2010 OpenStack LLC.
+# Copyright 2010 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -21,21 +21,18 @@
 """Utility methods for working with WSGI servers."""
 
 import re
-import socket
-import sys
 
-import eventlet.wsgi
 import routes.middleware
-import ssl
 import webob.dec
 import webob.exc
 
 from keystone.common import config
-from keystone.common import logging
 from keystone.common import utils
 from keystone import exception
+from keystone.openstack.common import gettextutils
 from keystone.openstack.common import importutils
 from keystone.openstack.common import jsonutils
+from keystone.openstack.common import log as logging
 
 
 CONF = config.CONF
@@ -77,98 +74,65 @@ def mask_password(message, is_unicode=False, secret="***"):
     return result
 
 
-class WritableLogger(object):
-    """A thin wrapper that responds to `write` and logs."""
+def validate_token_bind(context, token_ref):
+    bind_mode = CONF.token.enforce_token_bind
 
-    def __init__(self, logger, level=logging.DEBUG):
-        self.logger = logger
-        self.level = level
+    if bind_mode == 'disabled':
+        return
 
-    def write(self, msg):
-        self.logger.log(self.level, msg)
+    bind = token_ref.get('bind', {})
 
+    # permissive and strict modes don't require there to be a bind
+    permissive = bind_mode in ('permissive', 'strict')
 
-class Server(object):
-    """Server class to manage multiple WSGI sockets and applications."""
+    # get the named mode if bind_mode is not one of the known
+    name = None if permissive or bind_mode == 'required' else bind_mode
 
-    def __init__(self, application, host=None, port=None, threads=1000):
-        self.application = application
-        self.host = host or '0.0.0.0'
-        self.port = port or 0
-        self.pool = eventlet.GreenPool(threads)
-        self.socket_info = {}
-        self.greenthread = None
-        self.do_ssl = False
-        self.cert_required = False
+    if not bind:
+        if permissive:
+            # no bind provided and none required
+            return
+        else:
+            LOG.info(_("No bind information present in token"))
+            raise exception.Unauthorized()
 
-    def start(self, key=None, backlog=128):
-        """Run a WSGI server with the given application."""
-        LOG.debug(_('Starting %(arg0)s on %(host)s:%(port)s') %
-                  {'arg0': sys.argv[0],
-                   'host': self.host,
-                   'port': self.port})
+    if name and name not in bind:
+        LOG.info(_("Named bind mode %s not in bind information"), name)
+        raise exception.Unauthorized()
 
-        # TODO(dims): eventlet's green dns/socket module does not actually
-        # support IPv6 in getaddrinfo(). We need to get around this in the
-        # future or monitor upstream for a fix
-        info = socket.getaddrinfo(self.host,
-                                  self.port,
-                                  socket.AF_UNSPEC,
-                                  socket.SOCK_STREAM)[0]
-        _socket = eventlet.listen(info[-1],
-                                  family=info[0],
-                                  backlog=backlog)
-        if key:
-            self.socket_info[key] = _socket.getsockname()
-        # SSL is enabled
-        if self.do_ssl:
-            if self.cert_required:
-                cert_reqs = ssl.CERT_REQUIRED
-            else:
-                cert_reqs = ssl.CERT_NONE
-            sslsocket = eventlet.wrap_ssl(_socket, certfile=self.certfile,
-                                          keyfile=self.keyfile,
-                                          server_side=True,
-                                          cert_reqs=cert_reqs,
-                                          ca_certs=self.ca_certs)
-            _socket = sslsocket
+    for bind_type, identifier in bind.iteritems():
+        if bind_type == 'kerberos':
+            if not context.get('AUTH_TYPE', '').lower() == 'negotiate':
+                LOG.info(_("Kerberos credentials required and not present"))
+                raise exception.Unauthorized()
 
-        self.greenthread = self.pool.spawn(self._run,
-                                           self.application,
-                                           _socket)
+            if not context.get('REMOTE_USER') == identifier:
+                LOG.info(_("Kerberos credentials do not match those in bind"))
+                raise exception.Unauthorized()
 
-    def set_ssl(self, certfile, keyfile=None, ca_certs=None,
-                cert_required=True):
-        self.certfile = certfile
-        self.keyfile = keyfile
-        self.ca_certs = ca_certs
-        self.cert_required = cert_required
-        self.do_ssl = True
+            LOG.info(_("Kerberos bind authentication successful"))
 
-    def kill(self):
-        if self.greenthread:
-            self.greenthread.kill()
-
-    def wait(self):
-        """Wait until all servers have completed running."""
-        try:
-            self.pool.waitall()
-        except KeyboardInterrupt:
-            pass
-
-    def _run(self, application, socket):
-        """Start a WSGI server in a new green thread."""
-        log = logging.getLogger('eventlet.wsgi.server')
-        try:
-            eventlet.wsgi.server(socket, application, custom_pool=self.pool,
-                                 log=WritableLogger(log))
-        except Exception:
-            LOG.exception(_('Server error'))
-            raise
+        elif bind_mode == 'permissive':
+            LOG.debug(_("Ignoring unknown bind for permissive mode: "
+                        "{%(bind_type)s: %(identifier)s}"),
+                      {'bind_type': bind_type, 'identifier': identifier})
+        else:
+            LOG.info(_("Couldn't verify unknown bind: "
+                       "{%(bind_type)s: %(identifier)s}"),
+                     {'bind_type': bind_type, 'identifier': identifier})
+            raise exception.Unauthorized()
 
 
 class Request(webob.Request):
-    pass
+    def best_match_language(self):
+        """Determines the best available locale from the Accept-Language
+        HTTP header passed in the request.
+        """
+
+        if not self.accept_language:
+            return None
+        return self.accept_language.best_match(
+            gettextutils.get_available_languages('keystone'))
 
 
 class BaseApplication(object):
@@ -186,18 +150,18 @@ class BaseApplication(object):
 
             [app:wadl]
             latest_version = 1.3
-            paste.app_factory = nova.api.fancy_api:Wadl.factory
+            paste.app_factory = keystone.fancy_api:Wadl.factory
 
         which would result in a call to the `Wadl` class as
 
-            import nova.api.fancy_api
-            fancy_api.Wadl(latest_version='1.3')
+            import keystone.fancy_api
+            keystone.fancy_api.Wadl(latest_version='1.3')
 
         You could of course re-implement the `factory` method in subclasses,
         but using the kwarg passing it shouldn't be necessary.
 
         """
-        return cls()
+        return cls(**local_config)
 
     def __call__(self, environ, start_response):
         r"""Subclasses will probably want to implement __call__ like this:
@@ -237,24 +201,33 @@ class BaseApplication(object):
 
 
 class Application(BaseApplication):
-    @webob.dec.wsgify
+    @webob.dec.wsgify(RequestClass=Request)
     def __call__(self, req):
         arg_dict = req.environ['wsgiorg.routing_args'][1]
         action = arg_dict.pop('action')
         del arg_dict['controller']
         LOG.debug(_('arg_dict: %s'), arg_dict)
 
-        # allow middleware up the stack to provide context & params
+        # allow middleware up the stack to provide context, params and headers.
         context = req.environ.get(CONTEXT_ENV, {})
         context['query_string'] = dict(req.params.iteritems())
+        context['headers'] = dict(req.headers.iteritems())
         context['path'] = req.environ['PATH_INFO']
         params = req.environ.get(PARAMS_ENV, {})
-        if 'REMOTE_USER' in req.environ:
-            context['REMOTE_USER'] = req.environ['REMOTE_USER']
-        elif context.get('REMOTE_USER', None) is not None:
-            del context['REMOTE_USER']
+
+        for name in ['REMOTE_USER', 'AUTH_TYPE']:
+            try:
+                context[name] = req.environ[name]
+            except KeyError:
+                try:
+                    del context[name]
+                except KeyError:
+                    pass
+
         params.update(arg_dict)
 
+        context.setdefault('is_admin', False)
+
         # TODO(termie): do some basic normalization on methods
         method = getattr(self, action)
 
@@ -264,18 +237,21 @@ class Application(BaseApplication):
         try:
             result = method(context, **params)
         except exception.Unauthorized as e:
-            LOG.warning(_("Authorization failed. %s from %s")
-                        % (e, req.environ['REMOTE_ADDR']))
-            return render_exception(e)
+            LOG.warning(
+                _('Authorization failed. %(exception)s from %(remote_addr)s') %
+                {'exception': e, 'remote_addr': req.environ['REMOTE_ADDR']})
+            return render_exception(e, user_locale=req.best_match_language())
         except exception.Error as e:
             LOG.warning(e)
-            return render_exception(e)
+            return render_exception(e, user_locale=req.best_match_language())
         except TypeError as e:
-            logging.exception(e)
-            return render_exception(exception.ValidationError(e))
+            LOG.exception(e)
+            return render_exception(exception.ValidationError(e),
+                                    user_locale=req.best_match_language())
         except Exception as e:
-            logging.exception(e)
-            return render_exception(exception.UnexpectedError(exception=e))
+            LOG.exception(e)
+            return render_exception(exception.UnexpectedError(exception=e),
+                                    user_locale=req.best_match_language())
 
         if result is None:
             return render_response(status=(204, 'No Content'))
@@ -307,30 +283,30 @@ class Application(BaseApplication):
     def assert_admin(self, context):
         if not context['is_admin']:
             try:
-                user_token_ref = self.token_api.get_token(
-                    context=context, token_id=context['token_id'])
+                user_token_ref = self.token_api.get_token(context['token_id'])
             except exception.TokenNotFound as e:
                 raise exception.Unauthorized(e)
 
+            validate_token_bind(context, user_token_ref)
             creds = user_token_ref['metadata'].copy()
 
             try:
                 creds['user_id'] = user_token_ref['user'].get('id')
             except AttributeError:
-                logging.debug('Invalid user')
+                LOG.debug('Invalid user')
                 raise exception.Unauthorized()
 
             try:
                 creds['tenant_id'] = user_token_ref['tenant'].get('id')
             except AttributeError:
-                logging.debug('Invalid tenant')
+                LOG.debug('Invalid tenant')
                 raise exception.Unauthorized()
 
             # NOTE(vish): this is pretty inefficient
-            creds['roles'] = [self.identity_api.get_role(context, role)['name']
+            creds['roles'] = [self.identity_api.get_role(role)['name']
                               for role in creds.get('roles', [])]
             # Accept either is_admin or the admin role
-            self.policy_api.enforce(context, creds, 'admin_required', {})
+            self.policy_api.enforce(creds, 'admin_required', {})
 
 
 class Middleware(Application):
@@ -355,12 +331,12 @@ class Middleware(Application):
 
             [filter:analytics]
             redis_host = 127.0.0.1
-            paste.filter_factory = nova.api.analytics:Analytics.factory
+            paste.filter_factory = keystone.analytics:Analytics.factory
 
         which would result in a call to the `Analytics` class as
 
-            import nova.api.analytics
-            analytics.Analytics(app_from_paste, redis_host='127.0.0.1')
+            import keystone.analytics
+            keystone.analytics.Analytics(app, redis_host='127.0.0.1')
 
         You could of course re-implement the `factory` method in subclasses,
         but using the kwarg passing it shouldn't be necessary.
@@ -369,7 +345,7 @@ class Middleware(Application):
         def _factory(app):
             conf = global_config.copy()
             conf.update(local_config)
-            return cls(app)
+            return cls(app, **local_config)
         return _factory
 
     def __init__(self, application):
@@ -391,11 +367,24 @@ class Middleware(Application):
 
     @webob.dec.wsgify(RequestClass=Request)
     def __call__(self, request):
-        response = self.process_request(request)
-        if response:
-            return response
-        response = request.get_response(self.application)
-        return self.process_response(request, response)
+        try:
+            response = self.process_request(request)
+            if response:
+                return response
+            response = request.get_response(self.application)
+            return self.process_response(request, response)
+        except exception.Error as e:
+            LOG.warning(e)
+            return render_exception(e,
+                                    user_locale=request.best_match_language())
+        except TypeError as e:
+            LOG.exception(e)
+            return render_exception(exception.ValidationError(e),
+                                    user_locale=request.best_match_language())
+        except Exception as e:
+            LOG.exception(e)
+            return render_exception(exception.UnexpectedError(exception=e),
+                                    user_locale=request.best_match_language())
 
 
 class Debug(Middleware):
@@ -408,7 +397,7 @@ class Debug(Middleware):
 
     @webob.dec.wsgify(RequestClass=Request)
     def __call__(self, req):
-        if LOG.isEnabledFor(logging.DEBUG):
+        if not hasattr(LOG, 'isEnabledFor') or LOG.isEnabledFor(LOG.debug):
             LOG.debug('%s %s %s', ('*' * 20), 'REQUEST ENVIRON', ('*' * 20))
             for key, value in req.environ.items():
                 LOG.debug('%s = %s', key, mask_password(value,
@@ -420,7 +409,7 @@ class Debug(Middleware):
             LOG.debug('')
 
         resp = req.get_response(self.application)
-        if LOG.isEnabledFor(logging.DEBUG):
+        if not hasattr(LOG, 'isEnabledFor') or LOG.isEnabledFor(LOG.debug):
             LOG.debug('%s %s %s', ('*' * 20), 'RESPONSE HEADERS', ('*' * 20))
             for (key, value) in resp.headers.iteritems():
                 LOG.debug('%s = %s', key, value)
@@ -469,7 +458,7 @@ class Router(object):
         # if we're only running in debug, bump routes' internal logging up a
         # notch, as it's very spammy
         if CONF.debug:
-            logging.getLogger('routes.middleware').setLevel(logging.INFO)
+            logging.getLogger('routes.middleware')
 
         self.map = mapper
         self._router = routes.middleware.RoutesMiddleware(self._dispatch,
@@ -497,7 +486,8 @@ class Router(object):
         match = req.environ['wsgiorg.routing_args'][1]
         if not match:
             return render_exception(
-                exception.NotFound(_('The resource could not be found.')))
+                exception.NotFound(_('The resource could not be found.')),
+                user_locale=req.best_match_language())
         app = match['controller']
         return app
 
@@ -555,12 +545,12 @@ class ExtensionRouter(Router):
 
             [filter:analytics]
             redis_host = 127.0.0.1
-            paste.filter_factory = nova.api.analytics:Analytics.factory
+            paste.filter_factory = keystone.analytics:Analytics.factory
 
         which would result in a call to the `Analytics` class as
 
-            import nova.api.analytics
-            analytics.Analytics(app_from_paste, redis_host='127.0.0.1')
+            import keystone.analytics
+            keystone.analytics.Analytics(app, redis_host='127.0.0.1')
 
         You could of course re-implement the `factory` method in subclasses,
         but using the kwarg passing it shouldn't be necessary.
@@ -569,7 +559,7 @@ class ExtensionRouter(Router):
         def _factory(app):
             conf = global_config.copy()
             conf.update(local_config)
-            return cls(app)
+            return cls(app, **local_config)
         return _factory
 
 
@@ -591,12 +581,13 @@ def render_response(body=None, status=None, headers=None):
                           headerlist=headers)
 
 
-def render_exception(error):
+def render_exception(error, user_locale=None):
     """Forms a WSGI response based on the current error."""
     body = {'error': {
         'code': error.code,
         'title': error.title,
-        'message': str(error)
+        'message': unicode(gettextutils.get_localized_message(error.args[0],
+                                                              user_locale)),
     }}
     if isinstance(error, exception.AuthPluginException):
         body['error']['identity'] = error.authentication
