@@ -1,3 +1,6 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+# flake8: noqa
+
 # Copyright 2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -12,4 +15,4 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from keystone.contrib.admin_crud.core import *  # noqa
+from keystone.contrib.admin_crud.core import *
