@@ -1,6 +1,6 @@
 #!/bin/bash
 
-# Copyright 2012 OpenStack LLC
+# Copyright 2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -27,8 +27,9 @@ function usage {
   echo "  -x, --stop               Stop running tests after the first error or failure."
   echo "  -f, --force              Force a clean re-build of the virtual environment. Useful when dependencies have been added."
   echo "  -u, --update             Update the virtual environment with any newer package versions"
-  echo "  -p, --pep8               Just run pep8"
-  echo "  -P, --no-pep8            Don't run pep8"
+  echo "  -p, --pep8               Just run flake8"
+  echo "  -8, --8                  Just run flake8, don't show PEP8 text for each error"
+  echo "  -P, --no-pep8            Don't run flake8"
   echo "  -c, --coverage           Generate coverage report"
   echo "  -h, --help               Print this usage message"
   echo "  -xintegration            Ignore all keystoneclient test cases (integration tests)"
@@ -50,9 +51,9 @@ function process_option {
     -n|--no-recreate-db) recreate_db=0;;
     -f|--force) force=1;;
     -u|--update) update=1;;
-    -p|--pep8) just_pep8=1;;
-    -8|--8) short_pep8=1;;
-    -P|--no-pep8) no_pep8=1;;
+    -p|--pep8) just_flake8=1;;
+    -8|--8) short_flake8=1;;
+    -P|--no-pep8) no_flake8=1;;
     -c|--coverage) coverage=1;;
     -xintegration) nokeystoneclient=1;;
     --standard-threads)
@@ -69,11 +70,11 @@ always_venv=0
 never_venv=0
 force=0
 noseargs=
-noseopts=
+noseopts="--with-openstack --openstack-color"
 wrapper=""
-just_pep8=0
-short_pep8=0
-no_pep8=0
+just_flake8=0
+short_flake8=0
+no_flake8=0
 coverage=0
 nokeystoneclient=0
 recreate_db=1
@@ -90,9 +91,17 @@ fi
 
 if [ $nokeystoneclient -eq 1 ]; then
     # disable the integration tests
-    noseopts="$noseopts -I test_keystoneclient*"
+    noseopts="$noseopts -I test_keystoneclient* -I _test_import_auth_token.py"
 fi
 
+function cleanup_test_db {
+  # Default test settings will leave around some test*.db files
+  # TODO(termie): this could probably be moved into keystone/tests/__init__.py
+  #               but there have been some issues with creating that
+  #               file for some users
+  rm -f keystone/tests/*.db
+}
+
 function run_tests {
   # Just run the test suites in current environment
   ${wrapper} $NOSETESTS
@@ -109,25 +118,18 @@ function run_tests {
   return $RESULT
 }
 
-function run_pep8 {
+function run_flake8 {
   FLAGS=--show-pep8
-  echo $#
   if [ $# -gt 0 ] && [ 'short' == ''$1 ]
   then
       FLAGS=''
   fi
 
 
-  echo "Running pep8 ..."
-  # Opt-out files from pep8
-  ignore_scripts="*.pyc,*.pyo,*.sh,*.swp,*.rst"
-  ignore_files="*pip-requires"
-  ignore_dirs=".venv,.tox,dist,doc,openstack,vendor,*egg"
-  ignore="$ignore_scripts,$ignore_files,$ignore_dirs"
-  srcfiles="."
-  # Just run PEP8 in current environment
-  ${wrapper} pep8 --repeat $FLAGS --show-source \
-    --exclude=${ignore} ${srcfiles} | tee pep8.txt
+  echo "Running flake8 ..."
+  # Just run flake8 in current environment
+  echo ${wrapper} flake8 $FLAGS | tee pep8.txt
+  ${wrapper} flake8 $FLAGS | tee pep8.txt
 }
 
 NOSETESTS="nosetests $noseopts $noseargs"
@@ -167,30 +169,30 @@ if [ $coverage -eq 1 ]; then
     ${wrapper} coverage erase
 fi
 
-if [ $just_pep8 -eq 1 ]; then
-    run_pep8
+if [ $just_flake8 -eq 1 ]; then
+    run_flake8
     exit
 fi
 
-if [ $short_pep8 -eq 1 ]; then
-     run_pep8 short
+if [ $short_flake8 -eq 1 ]; then
+     run_flake8 short
      exit
 fi
 
 
 if [ $recreate_db -eq 1 ]; then
-    rm -f tests.sqlite
+    cleanup_test_db
 fi
 
 run_tests
 
-# NOTE(sirp): we only want to run pep8 when we're running the full-test suite,
-# not when we're running tests individually. To handle this, we need to
-# distinguish between options (noseopts), which begin with a '-', and
-# arguments (noseargs).
+# NOTE(sirp): we only want to run flake8 when we're running the full-test
+# suite, not when we're running tests individually. To handle this, we need to
+# distinguish between options (noseopts), which begin with a '-', and arguments
+# (noseargs).
 if [ -z "$noseargs" ]; then
-  if [ $no_pep8 -eq 0 ]; then
-    run_pep8
+  if [ $no_flake8 -eq 0 ]; then
+    run_flake8
   fi
 fi
 
